#include <iostream>
#include <algorithm>
#include <vector>
#include <utility>

using namespace std;

/*
 T Q
 R S
*/

double mul(const pair<int, int> &v, int s) {
    long long num = v.first * 1ll *  s;
    return num / double(v.second);
}

struct node {
    int _x1, _x2, _y1, _y2;
    pair<int, int> _v;
    node *T,*Q,*R,*S;
    bool _t, _q, _r, _s;
    node(int x1, int x2, int y1, int y2, pair<int, int> v = make_pair(0, 1)):
        _x1(x1), _x2(x2), _y1(y1), _y2(y2), _v(v),
        T(0), Q(0), R(0), S(0),
        _t(false), _q(false), _r(false), _s(false)
    {
    }
    void reset(pair<int, int> v) {
        _v = v;
        _t = _q = _r = _s = false;
    }
    void set(int x1, int x2, int y1, int y2, pair<int, int> v) {
        if(x1 <= _x1 && x2 >= _x2 && y1 <= _y1 && y2 >= _y2) {
            reset(v);
        } else {
            int mx = (_x1 + _x2) / 2;
            int my = (_y1 + _y2) / 2;
            if(x1 <= mx && x2 >= _x1 && y1 <= my && y2 >= _y1) {
                if(!T) T = new node(_x1, mx, _y1, my, _v);
                if(!_t) {
                    T->reset(_v);
                    _t = true;
                }
                T->set(x1, x2, y1, y2, v);
            }
            if(x1 <= mx && x2 >= _x1 && y1 <= _y2 && y2 > my) {
                if(!R) R = new node(_x1, mx, my + 1, _y2, _v);
                if(!_r) {
                    R->reset(_v);
                    _r = true;
                }
                R->set(x1, x2, y1, y2, v);
            }
            if(x1 <= x2 && x2 > mx && y1 <= my && y2 >= _y1) {
                if(!Q) Q = new node(mx + 1, _x2, _y1, my, _v);
                if(!_q) {
                    Q->reset(_v);
                    _q = true;
                }
                Q->set(x1, x2, y1, y2, v);
            }
            if(x1 <= x2 && x2 > mx && y1 <= _y2 && y2 > my) {
                if(!S) S = new node(mx + 1, _x2, my + 1, _y2, _v);
                if(!_s) {
                    S->reset(_v);
                    _s = true;
                }
                S->set(x1, x2, y1, y2, v);
            }
        }
    }
    double sum() {
        int mx = (_x1 + _x2) / 2;
        int my = (_y1 + _y2) / 2;

        double s = 0.0;
        s += _t ? T->sum() : mul(_v, (mx - _x1 + 1) * (my - _y1 + 1));
        s += _q ? Q->sum() : mul(_v, (_x2 - mx) * (my - _y1 + 1));
        s += _r ? R->sum() : mul(_v, (mx - _x1 + 1) * (_y2 - my));
        s += _s ? S->sum() : mul(_v, (_x2 - mx) * (_y2 - my));

        return s;
    }
};

int main(int argc, char **argv) {
    int a, b;
    cin >> a >> b;

    auto tree = new node(0, a, 0, b);

    struct rec {
        int x1, x2, y1, y2, c;
    };

    int n; cin >> n;
    vector<rec> v;
    for(int i = 0; i < n; ++i) {
        int r, c, h, w, p;
        cin >> r >> c >> h >> w >> p;
        v.push_back({r, r + h - 1, c, c + w - 1, p});
    }

    sort(v.begin(), v.end(), [](const auto &a, const auto &b) {
        int a_s = (a.x2 - a.x1 + 1) * (a.y2 - a.y1 + 1);
        int b_s = (b.x2 - b.x1 + 1) * (b.y2 - b.y1 + 1);
        return a.c * 1ll * b_s < b.c * 1ll * a_s;
    });

    for(auto &u : v) {
        tree->set(u.x1, u.x2, u.y1, u.y2, make_pair(u.c, (u.x2 - u.x1 + 1) * (u.y2 - u.y1 + 1)));
    }

    cout.precision(20);
    cout << tree->sum() << endl;

    return 0;
}
