#include <iostream>
#include <cstdio>
#include <algorithm>
#include <vector>

using namespace std;

struct node
{
    int _x1, _y1, _x2, _y2;
    node *P, *Q, *R, *S;
    bool is_assigned;
    double sum;
    int occupied_cells;

    node(int __x1, int __y1, int __x2, int __y2)
    {
        _x1 = __x1;
        _y1 = __y1;
        _x2 = __x2;
        _y2 = __y2;
        P = Q = R = S = nullptr;
        is_assigned = false;
        sum = 0;
        occupied_cells = 0;
    }

    void set(int x1, int y1, int x2, int y2, double v)
    {
        if (is_assigned) {
            return;  // previous values were better
        }
        if (x2 < _x1 || _x2 < x1 || y2 < _y1 || _y2 < y1) {
            return;  // no intersection
        }
        if (x1 <= _x1 && _x2 <= x2 && y1 <= _y1 && _y2 <= y2) {
            is_assigned = true;
            auto area = (_x2 - _x1 + 1) * (_y2 - _y1 + 1);
            sum += v * (area - occupied_cells);
            occupied_cells = area;
            return;
        }

        int mx = (_x1 + _x2) / 2;
        int my = (_y1 + _y2) / 2;

        if (not P) {
            P = new node(_x1, _y1, mx, my);
        }
        P->set(x1, y1, x2, y2, v);

        if (mx + 1 <= _x2) {
            if (not Q) {
                Q = new node(mx + 1, _y1, _x2, my);
            }
            Q->set(x1, y1, x2, y2, v);
        }

        if (my + 1 <= _y2) {
            if (not R) {
                R = new node(_x1, my + 1, mx, _y2);
            }
            R->set(x1, y1, x2, y2, v);
        }

        if (mx + 1 <= _x2 and my + 1 <= _y2) {
            if (not S) {
                S = new node(mx + 1, my + 1, _x2, _y2);
            }
            S->set(x1, y1, x2, y2, v);
        }

        occupied_cells = 0;
        occupied_cells += P ? P->occupied_cells : 0;
        occupied_cells += Q ? Q->occupied_cells : 0;
        occupied_cells += R ? R->occupied_cells : 0;
        occupied_cells += S ? S->occupied_cells : 0;
        sum = 0.0;
        sum += P ? P->sum : 0.0;
        sum += Q ? Q->sum : 0.0;
        sum += R ? R->sum : 0.0;
        sum += S ? S->sum : 0.0;
    }
};

int m, n, k;

struct rect
{
    int x1, y1, x2, y2, p;
    int area;
};
vector<rect> R;

int main()
{
    ios_base::sync_with_stdio(false);

    cin >> m >> n >> k;
    auto root = new node(0, 0, m - 1, n - 1);
    R.resize(k);
    for (int i = 0 ;i < k; i++) {
        int r, c, h, w;
        cin >> r >> c >> h >> w >> R[i].p;
        R[i].x1 = r; R[i].y1 = c;
        R[i].x2 = r + h - 1; R[i].y2 = c + w - 1;
        R[i].area = h * w;
    }
    sort(R.begin(), R.end(), [](const auto &a, const auto &b) -> bool {
        return a.p * 1ll * b.area > b.p * 1ll * a.area;
    });
    for (auto &r : R) {
        root->set(r.x1, r.y1, r.x2, r.y2, r.p * 1.0 / r.area);
    }

    cout.setf(ios::fixed);
    cout.precision(10);
    cout << root->sum;

    return 0;
}
