#include <iostream>
#include <cstdlib>
#include <vector>
#include <set>
#include <map>
#include <sstream>
#include <string>
#include <limits>
#include "subprocess.h"
#include "tractor.h"
#include <unistd.h>
#include <unordered_set>
#include <unordered_map>

namespace
{
    constexpr int MAX_TRACTOR_COUNT = 100;
    constexpr int TURN_LIMIT = 1000;

    struct pairhash {
    public:
    template <typename T, typename U>
    std::size_t operator()(const std::pair<T, U> &x) const
      {
        return std::hash<T>()(x.first) ^ std::hash<U>()(x.second);
      }
    };

    auto check_state(const std::vector<TractorState> &tractors)
    {
        for (const auto &c : tractors) {
            if (c.x() > 0) {
                return false;
            }
        }
        return true;
    }

    void dfs(std::pair<int, int> u, std::unordered_set<std::pair<int, int>, pairhash> &used, std::unordered_map<std::pair<int, int>, std::vector<std::pair<int, int>>, pairhash> &graph) {
	used.insert(u);
	if (graph.count(u) == 0) {
	    return;
	}
	for (auto v : graph[u]) {
	     if (used.count(v) == 0) {
		 dfs(v, used, graph);
	     }
	}
    }

    void filter_moves(std::vector<TractorState> &tractors) {
	std::unordered_set<std::pair<int, int>, pairhash> used;
	std::unordered_map<std::pair<int, int>, std::vector<std::pair<int, int>>, pairhash> graph;
	std::unordered_map<std::pair<int, int>, TractorState*, pairhash> tractors_map;

        for (auto &t: tractors)	{
	    // new -> old
	    graph[std::make_pair(t.new_x(), t.new_y())].push_back(std::make_pair(t.x(), t.y()));
	    tractors_map[std::make_pair(t.x(), t.y())] = &t;
	}

	for (auto & [u, vv] : graph) {
	    if (vv.size() > 1) {
		dfs(u, used, graph);
	    }
	}

	for (auto u : used) {
		if (tractors_map[u]) {
			tractors_map[u]->reset_new_state();
		}
	}
    }
}

using std::cin;
using std::cout;
using std::cerr;
using std::endl;
using std::vector;
using std::set;
using std::map;
using std::pair;
using std::ostringstream;
using std::string;

int main(int argc, char *argv[])
{
    /*
    if (argc != 2) {
        cerr << "wrong number of arguments" << endl;
        exit(1);
    }
    string copterfile(argv[1]);
    cerr << "copter: " << copterfile << endl;
    */

    // read the copter count
    int tractor_count = 0;
    cin >> tractor_count;
    if (!cin || tractor_count <= 0 || tractor_count > MAX_TRACTOR_COUNT) {
        cerr << "invalid tractors count" << endl;
        exit(1);
    }
    // read the original state (x, y)
    vector<TractorState> tractors;
    for (int i = 0; i < tractor_count; ++i) {
        int x = 0, y = 0;
        cin >> x >> y;
        if (!cin) {
            cerr << "failed to read initial copter position" << endl;
            exit(1);
        }
        if (x < 1) {
            cerr << "incorrect initial tractor points" << endl;
            exit(1);
        }
        tractors.emplace_back(x, y);
    }
    // FIXME: the turn limit must be settable
    int turn_limit = TURN_LIMIT;

    int current_turn = 0;
    int retval = 0;

    // movements
    while (1) {
        ++current_turn;
        cerr << "Turn: " << current_turn << endl;

        cerr << "States:" << endl;
        for (int i = 0; i < tractor_count; ++i) {
            cerr << "    " << i << ": " << tractors[i].x() << " " << tractors[i].y() << " ";
            tractors[i].dump_mem(cerr);
        }

        if (current_turn > turn_limit) {
            cerr << "turn limit of " << turn_limit << " exceeded" << endl;
            current_turn = -2;
            break;
        }

        vector<TractorState> new_tractors(tractor_count);
        int failure_count = 0;
        for (int i = 0; i < tractor_count; ++i) {
            cerr << "    Tractor: " << i << endl;
            auto &c = tractors[i];
            // prepare tractor input
            ostringstream cfg;
            cfg << c.x() << " " << c.y() << endl;
            c.dump_mem(cfg);
            string tractor_input = cfg.str();
            cerr << "    Input: <" << tractor_input << ">" << endl;

            /*
            for (int j = 0; argv[j]; ++j) {
                cerr << "<" << argv[j] << ">" << endl;
            }
            */
            Subprocess prc;
            prc.set_cmd(argv + 1);
            //prc.set_cmd(copterfile);
            prc.set_input(std::move(tractor_input));
            prc.set_max_cpu_time(1);
            bool success = prc.run_and_wait();
            if (!success) {
                cerr << "tractor failed" << endl;
                cerr << "    Errors: <" << prc.move_error() << ">" << endl;
                ++failure_count;
                continue;
            }
            string tractor_output = prc.move_output();
            cerr << "    Output: <" << tractor_output << ">" << endl;
            cerr << "    Errors: <" << prc.move_error() << ">" << endl;
            try {
                c.parse_tractor_output(tractor_output);
            } catch (const std::exception &ex) {
                cerr << ex.what() << endl;
                ++failure_count;
                continue;
            }
        }

        if (failure_count > 0) {
            cerr << "Turn " << current_turn << " unsuccessful: " << failure_count << " tractors crashed" << endl;
            current_turn = -1;
            break;
        }

        filter_moves(tractors);

        for (auto &c : tractors) {
            c.apply_move();
	    cerr << "    New position: " << c.x() << ", " << c.y() << endl;
        }
        if (check_state(tractors)) {
            cerr << "target configuration achieved in " << current_turn << " turns" << endl;
            for (int i = 0; i < tractor_count; ++i) {
                cerr << "    " << i << ": " << tractors[i].x() << " " << tractors[i].y() << " ";
                tractors[i].dump_mem(cerr);
            }
            break;
        }

        cerr << "Turn " << current_turn << " ended" << endl;
        //sleep(1);
    }

    cout << current_turn << endl;

    return retval;
}
