#include <iostream>
#include <vector>
#include <algorithm>

#include <sys/types.h>
#include <sys/stat.h>
#include <fcntl.h>
#include <unistd.h>
#include <string.h>

static constexpr int MEMORY_SIZE = 8;

enum class Direction { AWAY = 0, LEFT = 1, RIGHT = 2, UP = 3, DOWN = 4, SAME = 5 };

using std::cin;
using std::cerr;
using std::cout;
using std::endl;
using std::vector;
using std::sort;

struct Position
{
    int x{}, y{};
};

struct TractorState
{
    Direction dir{Direction::AWAY};
    int x, y;
    unsigned char mem[MEMORY_SIZE]{};
};

unsigned get_value(unsigned char *mem)
{
    return mem[0] | (mem[1] << 8) | (mem[2] << 16) | (mem[3] << 24);
}

void random_bytes(unsigned char *mem, size_t size)
{
    static int fd = -1;
    if (fd < 0) {
        fd = open("/dev/urandom", O_RDONLY, 0);
        if (fd < 0) {
            cerr << "cannot open /dev/urandom" << endl;
            abort();
        }
    }
    ssize_t sz = read(fd, mem, size);
    if (sz < 0) {
        cerr << "read error from /dev/urandom" << endl;
        abort();
    }
    if (!sz) {
        cerr << "EOF from /dev/urandom" << endl;
        abort();
    }
    if (sz != (ssize_t) size) {
        cerr << "invalid read from /dev/urandom" << endl;
        abort();
    }
}


    
int main()
{
    TractorState self{};
   
    cin >> self.x >> self.y;

    for (int i = 0; i < MEMORY_SIZE; ++i) {
        int m;
        cin >> m;
        if (m < 0 || m > 255) {
            cerr << "invalid memory value" << endl;
            abort();
        }
        self.mem[i] = m;
    }

    if (self.mem[0] == 0) {
        // initial state
        self.mem[0] = 1;
	if (self.x == 1 && self.y == 1) {
             self.dir = Direction::RIGHT;
	} else {
	     self.dir = Direction::LEFT;
	}
    } else {
            self.dir = Direction::RIGHT;
    }

    if (self.dir == Direction(0)) {
        cerr << "copter does not make a turn" << endl;
        abort();
    }

    std::cout << int(self.dir);
    for (int i = 0; i < MEMORY_SIZE; ++i) {
        std::cout << " " << int(self.mem[i]);
    }
    cout << endl;
}
