#include <iostream>
#include <cstdlib>
#include <vector>
#include <set>
#include <map>
#include <sstream>
#include <string>
#include <limits>
#include <unistd.h>
#include <unordered_set>
#include <unordered_map>

    constexpr int MAX_TRACTOR_COUNT = 20;
    constexpr int TURN_LIMIT = 1000;

using namespace std;

class TractorState
{
public:
    static constexpr int MEMORY_SIZE = 8;
    // relative pos
    enum class Direction { AWAY = 0, LEFT = 1, RIGHT = 2, UP = 3, DOWN = 4, SAME = 5 };

private:
    int x_ = 0;
    int y_ = 0;

    int new_x_ = 0;
    int new_y_ = 0;

public:
    TractorState() = default;
    TractorState(int x, int y) : x_(x), y_(y) {}

    int x() const noexcept { return x_; }
    int y() const noexcept { return y_; }

    int new_x() const { return new_x_; }
    int new_y() const { return new_y_; }

    void parse_tractor_output(int dest) {
    if (dest <= 0 || dest > int(TractorState::Direction::SAME)) {
        throw std::invalid_argument(std::string("invalid destination ") + std::to_string(dest));
    }

    std::pair<int, int> new_point;

    // check for garbage
    switch (dest) {
    case 1: // left
        new_point = {x_ - 1, y_};
        break;
    case 2: // right
        new_point = {x_ + 1, y_};
        break;
    case 3: // up
        new_point = {x_, y_ + 1};
        break;
    case 4: // down
        new_point = {x_, y_ - 1};
        break;
    case 5: // stay put
        new_point = {x_, y_};
        break;
    default:
        throw std::invalid_argument(std::string("invalid destination ") + std::to_string(dest));
    }

    if (auto [x, y] = new_point; x == 0 && y != 0) {
        new_x_ = x_;
	new_y_ = y_;
    } else {
        new_x_ = x;
        new_y_ = y;
    }
    }

    void apply_move()
    {
        x_ = new_x_;
        y_ = new_y_;
    }

    void reset_new_state()
    {
        new_x_ = x_;
        new_y_ = y_;
    }
};

    struct pairhash {
    public:
    template <typename T, typename U>
    std::size_t operator()(const std::pair<T, U> &x) const
      {
        return std::hash<T>()(x.first) ^ std::hash<T>()(x.second);
        //return std::hash<T>()(x.first  * 1000000 + x.second);
      }
    };

    auto check_state(const std::vector<TractorState> &tractors)
    {
        for (const auto &c : tractors) {
            if (c.x() > 0) {
                return false;
            }
        }
        return true;
    }

    void dfs(std::pair<int, int> u, std::unordered_set<std::pair<int, int>, pairhash> &used, std::unordered_map<std::pair<int, int>, std::vector<std::pair<int, int>>, pairhash> &graph) {
	used.insert(u);
	cerr << u.first << ":" << u.second << endl;
        if (graph.count(u) == 0) {
            return;
        }
	for (auto v : graph[u]) {
	     if (used.count(v) == 0) {
		 dfs(v, used, graph);
	     }
	}
    }

    void filter_moves(std::vector<TractorState> &tractors) {
	std::unordered_set<std::pair<int, int>, pairhash> used;
	std::unordered_map<std::pair<int, int>, std::vector<std::pair<int, int>>, pairhash> graph;
	std::unordered_map<std::pair<int, int>, std::vector<TractorState*>, pairhash> tractors_map;

        for (auto &t: tractors)	{
	    // new -> old
	    graph[std::make_pair(t.new_x(), t.new_y())].push_back(std::make_pair(t.x(), t.y()));
	    tractors_map[std::make_pair(t.new_x(), t.new_y())].push_back(&t);
	}

	for (auto & [u, vv] : graph) {
            cerr << u.first << "-" << u.second << endl;		
	    if (vv.size() > 1) {
		cerr << u.first << "+" << u.second << endl;
		dfs(u, used, graph);
	    }
	}

	for (auto u : used) {
		cerr << "u " << u.first << " " << u.second << endl;
		for (auto ptractor : tractors_map[u]) {
			ptractor->reset_new_state();
		}
	}
    }

using std::cin;
using std::cout;
using std::cerr;
using std::endl;
using std::vector;
using std::set;
using std::map;
using std::pair;
using std::ostringstream;
using std::string;

int main(int argc, char *argv[])
{
    /*
    if (argc != 2) {
        cerr << "wrong number of arguments" << endl;
        exit(1);
    }
    string copterfile(argv[1]);
    cerr << "copter: " << copterfile << endl;
    */

    // read the copter count
    int tractor_count = 0;
    cin >> tractor_count;
   // read the original state (x, y)
    vector<TractorState> tractors;
    for (int i = 0; i < tractor_count; ++i) {
        int x = 0, y = 0, dest = 0;
        cin >> x >> y >> dest;
        if (!cin) {
            cerr << "failed to read initial copter position" << endl;
            exit(1);
        }
        if (x == 0 && y != 0) {
            cerr << "incorrect initial tractor points" << endl;
            exit(1);
        }
        tractors.emplace_back(x, y);
	tractors.back().parse_tractor_output(dest);
    }

    filter_moves(tractors);

        for (auto &c : tractors) {
            c.apply_move();
	    cout << c.x() << " " << c.y() << endl;
        }
    return 0;
}
