#include "tractor.h"

#include <sstream>
#include <exception>
#include <cstring>

void
TractorState::dump_mem(std::ostream &out) const
{
    for (int i = 0; i < MEMORY_SIZE; ++i) {
        if (i > 0) out << ' ';
        out << int(mem_[i]);
    }
    out << std::endl;
}

void
TractorState::parse_tractor_output(std::string &str)
{
    std::istringstream ss(str);
    int dest = 0;
    ss >> dest;
    if (!ss || dest <= 0 || dest > int(TractorState::Direction::SAME)) {
        throw std::invalid_argument(std::string("invalid destination ") + std::to_string(dest));
    }
    unsigned char mem[MEMORY_SIZE] = {};
    for (int i = 0; i < MEMORY_SIZE; ++i) {
        int val;
        ss >> val;
        if (!ss) {
            throw std::invalid_argument(std::string("failed to read memory value ") + "[" + std::to_string(i) + "]");
        }
        if (val < 0 || val > 255) {
            throw std::invalid_argument(std::string("invalid memory value ") + "[" + std::to_string(i) + "]: " + std::to_string(val));
        }
        mem[i] = val;
    }

    memcpy(mem_, mem, sizeof(mem_));

    std::pair<int, int> new_point;

    // check for garbage
    switch (dest) {
    case 1: // left
        new_point = {x_ - 1, y_};
        break;
    case 2: // right
        new_point = {x_ + 1, y_};
        break;
    case 3: // up
        new_point = {x_, y_ + 1};
        break;
    case 4: // down
        new_point = {x_, y_ - 1};
        break;
    case 5: // stay put
        new_point = {x_, y_};
        break;
    default:
        throw std::invalid_argument(std::string("invalid destination ") + std::to_string(dest));
    }

    if (auto [x, y] = new_point; x == 0 && y != 0) {
        new_x_ = x_;
	new_y_ = y_;
    } else {
        new_x_ = x;
        new_y_ = y;
    }
}


